
############################################################
#                                                          #
#                 Loop Universal Kriging                   #
#                                                          #
############################################################
#Loop to apply LOOCV to Caliope data: 


compute_in_UTM        <- TRUE #We want to transform the coordinates from CSR to UTM, it is faster in Universal Kriging
crs_model_utm         <- "+proj=utm +zone=31 +ellps=intl +units=m +no_defs"
crs_model             <- "+proj=longlat +datum=WGS84 +ellps=WGS84 +towgs84=0,0,0" #coordinates sistem

#LOOCV_total and llista_uk_total is created empty to fill it later with LOOCV and raw model data respectivelly
LOOCV_total           <- matrix(NA,ncol=nrow(local_stations@data),nrow=0) #list(length=nrow(date_times)) #we will have a large list with a data.frame for each hour
colnames(LOOCV_total) <- local_stations$ref_obs #we want a matrix with empty data and the name of stations

llista_uk_total           <- matrix(NA,ncol=nrow(local_stations@data),nrow=0) #list(length=nrow(date_times)) #we will have a large list with a data.frame for each hour
colnames(llista_uk_total) <- local_stations$ref_obs #we want a matrix with empty data and the name of stations

#starting OK and LOOCV:
for (hour in 1:nrow(date_times)) {
  print(paste0("loop UK number:",hour))
  df_model <- model[,,hour]
  
  #if all the model for a certain hour is nan we cannot apply UK
  if ( sum(is.nan(df_model))!=length(df_model)  ){
    
    #print(df_obs_all[hour,1])
    df_obs   <- df_obs_sub[hour,-1]
    
    #sum(is.na(df_obs)). Hem de marcar-nos fins a quantes observacions fem l'UK, i en cas de no ternir-ne suficients si agafar un variograma del UK previ o bé saltar a l'else
    
    # 1. Interpolate model values at obs locations
    #    1.1. Set spatialpointsdataframe
    #raw model:
    df_model                  <- data.frame(lat=lat_model,lon=lon_model,NO2=sapply(df_model,as.matrix))
    coordinates(df_model)     <- ~lon+lat
    sp::proj4string(df_model) <- crs_model
    
    #config the coordinates in UTM
    if (compute_in_UTM) {
      df_model <- spTransform(df_model, sp::CRS(crs_model_utm))
    }
    
    #observations:
    df_obs                     <- data.frame(lat=local_stations@coords[,2],lon=local_stations@coords[,1],NO2=as.numeric(df_obs[1,]),code=colnames(df_obs))
    df_obs                     <- na.omit(df_obs)
    df_obs[df_obs$NO2<0,"NO2"] <- NA
    df_obs                     <- na.omit(df_obs)
    coordinates(df_obs)        <- ~lon+lat
    sp::proj4string(df_obs)    <- crs_model
    
    #config the coordinates in UTM
    if (compute_in_UTM) {
      df_obs <- spTransform(df_obs, sp::CRS(crs_model_utm))
    }
    
    
    #    1.2 Compute Variogram
    set.seed(1) #we are picking random points in the next line, set.seed permits us to have same points in each loop
    pick_random_points  <- sample(1:nrow(df_model),2000)
    NO2.fit_ok          <- automap::autofitVariogram(formula=NO2~1, input_data=df_model[pick_random_points,],model="Ste")
    # Hem posat model='Ste' pk la segona hora del 01/06/2021 peta el variograma, mirar bé perque no s'ajusa
    
    #    1.3 Compute ordinary Kriging
    
    #Del model interpolo cap a les coordenades de les obs 'OK'
    ok_gstat <- gstat(formula=NO2~1, data = df_model, model = NO2.fit_ok$var_model, nmax=100)
    
    #Interpolacio del model a les coordenades del conjunt
    OK       <- predict(object=ok_gstat, newdata=df_obs)
    
    #If the variogram has singular matrix then it produces NAN's, so in that case we take a good Variogram instead:
    if(any(is.na(OK@data$var1.pred))){
      print("WARNING: VARIOGRAM DIVERGED! Using a precalculated VARIOGRAM")
      NO2.fit_ok=readRDS(variogram_ok)
      ok_gstat<- gstat(formula=NO2~1, data = df_model, model = NO2.fit_ok$var_model, nmax=100)
      
      #Interpolacio del model a les coordenades del conjunt
      OK <-predict(object=ok_gstat, newdata=df_obs)
    }
    
    # Arrange data obs-raw_model
    llista_uk               <- data.frame(df_obs@coords)
    llista_uk$NO2_obs       <- df_obs@data$NO2
    llista_uk$NO2_model     <- OK$var1.pred
    coordinates(llista_uk)  <- ~lon+lat
    
    #config the coordinates in UTM
    if (compute_in_UTM) {
      sp::proj4string(llista_uk) <- crs_model_utm
    } else {
      sp::proj4string(llista_uk) <- crs_model
    }
    
    # Compute UK variogram
    
    # in exceptional cases we have problems with autofitVariogram (eg: date="20211108" at 18 h) we still don't know why (maybe few points to execute the variogram, also we see negative range...)
    
    NO2.fit_uk <- automap::autofitVariogram(formula=NO2_obs~NO2_model,input_data=llista_uk, verbose=FALSE)
    
    #Emergency Variogram if range of the current Variogram it's negative
    if(any(NO2.fit_uk$var_model$range<0)){
      print('UK Variogram had negative range')
      NO2.fit_uk=readRDS(variogram_uk)
    }
    
    # LOOCV
    ok_loocv             <- krige.cv(NO2_obs~NO2_model,llista_uk,NO2.fit_uk$var_model)
    
    # To check everything is fine, compute UK using all stations
    #uk_gstat<- gstat(formula=NO2_obs~NO2_model,data=llista_uk, model = NO2.fit_uk$var_model)
    #ok_loocv <-predict(object=uk_gstat, newdata=llista_uk)
    
    
    
    # dtaframe amb columnes df_obs$code ajuntar LOOCV based on code
    
    # Save LOOCV and raw_model
    uk_loocv_h           <- matrix(data=ok_loocv@data$var1.pred, nrow=1)
    colnames(uk_loocv_h) <- df_obs$code
    
    llista_uk_h           <- matrix(data=llista_uk@data$NO2_model, nrow=1)
    colnames(llista_uk_h) <- df_obs$code
    
    llista_uk_total <-  plyr::rbind.fill(as.data.frame(llista_uk_total), as.data.frame(llista_uk_h))
    LOOCV_total     <-  plyr::rbind.fill(as.data.frame(LOOCV_total),as.data.frame(uk_loocv_h))
    
  }else{
    
    uk_loocv_h            <- matrix(data=NA, nrow=1,ncol=(length(df_obs_all)-1))
    colnames(uk_loocv_h)  <- colnames(df_obs_all[,-1])
    LOOCV_total           <- plyr::rbind.fill(LOOCV_total,as.data.frame(uk_loocv_h))
    
    llista_uk_h           <- matrix(data=NA, nrow=1,ncol=(length(df_obs_all)-1))
    colnames(llista_uk_h) <- colnames(df_obs_all[,-1])
    llista_uk_total       <- plyr::rbind.fill(llista_uk_total,as.data.frame(llista_uk_h))
    
  }
}

LOOCV_total_df       <- as.data.frame(LOOCV_total)
LOOCV_total_df$dates <- date_times$date

llista_uk_total_df       <- as.data.frame(llista_uk_total)
llista_uk_total_df$dates <- date_times$date

#Arrange LOOCV_total_df to remove negative values:

LOOCV_total_df_wd        <- LOOCV_total_df %>% dplyr::select(-dates)
# Realment es podria fer abans de juntar el date al sortir del bucle. Ho deixo així perquè tenim LOOCV guardat amb la data
dates                    <- LOOCV_total_df %>% dplyr::select(dates)  

for(i in 1:nrow(local_stations)){
  LOOCV_total_df_wd[[i]] <- replace(LOOCV_total_df[[i]], LOOCV_total_df[[i]]<0, 0)
}

LOOCV_total_df <- cbind(LOOCV_total_df_wd,dates)

# Verification of negative values 'which(llista_uk_total_df<0)'. At the moment we don't apply it to llista_uk_total because it doesn't have negative values


saveRDS(object=LOOCV_total_df, file=paste0(save_result,"/LOOCV_total_df.rds"))
saveRDS(object=llista_uk_total_df, file=paste0(save_result,"/llista_uk_total_df.rds"))

