
############################################################
#                                                          #
#              evaluation statistics                       #
#                                                          #
############################################################

# it alerts us if the dates in model do not coincide with the dates of the obs
if ( any(date_times$date != df_obs_sub$date))  {
  print("ERROR dates from model and observations are wrong")
}

#-------------------------------------------------------------------------------
#We need a data.frame with raw_model and LOOCV_model to apply the stadistics

# Comparation using all stations concatenated:
raw_model_total       <- NULL
corr_model_total      <- NULL 

for (i in 1:nrow(local_stations)) {
  
  raw_model_estacio <- data.frame(obs = df_obs_sub[ ,colnames(llista_uk_total)[i]],
                                  mod = llista_uk_total_df[ ,colnames(llista_uk_total)[i]],
                                  model="raw_model")
  
  corr_model_estacio <- data.frame(obs = df_obs_sub[ ,colnames(llista_uk_total)[i] ],
                                   mod = LOOCV_total_df[ ,colnames(llista_uk_total)[i]],
                                   model="corr_model")
  #raw_model_total and corr_model_total are a data frame with three lists (obs, mod, model(raw or corrected))
  raw_model_total   <- rbind(raw_model_total,raw_model_estacio) 
  corr_model_total  <- rbind(corr_model_total,corr_model_estacio)
}

mod_obs_data <- rbind(raw_model_total,
                      corr_model_total)

#Openair function that gives us the stadistics we want

Summary_stats <- openair::modStats(mod_obs_data, obs = "obs", mod = "mod", type = "model")
png(file = paste0(save_general_statistics,"All_stations","_stats.png"), width = 1472, height = 540)
gridExtra::grid.table(Summary_stats)
dev.off()

print("Statistic for all stations concateneited: ")

#-------------------------------------------------------------------------------
# Mean value for each hour all stations:
obs_total_mean          <- matrix(nrow = length(date_times$date), ncol = 0) 
raw_model_total_mean    <- matrix(nrow = length(date_times$date), ncol = 0)
corr_model_total_mean   <- matrix(nrow = length(date_times$date), ncol = 0)

for (i in 1:nrow(local_stations)) {
  obs_estacio        <-  df_obs_sub[ ,colnames(llista_uk_total)[i]]
  raw_model_estacio  <-  llista_uk_total_df[ ,colnames(llista_uk_total)[i]]
  corr_model_estacio <-  LOOCV_total_df[ ,colnames(llista_uk_total)[i]]
  
  obs_total_mean        <- cbind(obs_total_mean,obs_estacio)
  raw_model_total_mean  <- cbind(raw_model_total_mean,raw_model_estacio)
  corr_model_total_mean <- cbind(corr_model_total_mean,corr_model_estacio)
}

raw_model_mean      <- data.frame( obs = rowMeans(obs_total_mean,na.rm=TRUE),
                                   mod = rowMeans(raw_model_total_mean,na.rm=TRUE),
                                   model="raw_model")

corr_model_mean     <- data.frame( obs = rowMeans(obs_total_mean,na.rm=TRUE),
                                   mod = rowMeans(corr_model_total_mean,na.rm=TRUE),
                                   model="corr_model")

mod_obs_data_mean   <- rbind(raw_model_mean,
                             corr_model_mean)


Summary_stats <- openair::modStats(mod_obs_data_mean, obs = "obs", mod = "mod", type = "model")
print(Summary_stats)

png(file = paste0(save_general_statistics,"All_stations_hourly","_stats.png"), width = 1472, height = 540)
gridExtra::grid.table(Summary_stats)
dev.off()

print("Statistic considering the mean hour for all stations: ")

#-------------------------------------------------------------------------------
# Per una estació:
# First, we will evalue the raw model. To do so, we need a data frame with obs and the raw model. Obv we choose df_obs_sub as obs
# colnames(llista_uk_total)[1] --> We do the stadistics for each station
# 
# raw_model <- data.frame( obs = df_obs_sub[ ,colnames(llista_uk_total)[1] ],
#                          mod = llista_uk_total_df[ ,colnames(llista_uk_total)[1]],
#                          model="raw_model")
# 
# #Aqui posarem el LOOC_total_df  que s'ha anat generant en el bucle amb el model corretgit
# corr_model <- data.frame( obs = df_obs_sub[ ,colnames(llista_uk_total)[1] ],
#                           mod = LOOCV_total_df[ ,colnames(llista_uk_total)[1]],
#                           model="corr_model")
# 
# #tres columnes: obs, mod i quin tipus de model es 'model': que pot ser raw_model o corr_model. Primer et posa el raw i seguidament el corr
# mod_obs_data_1 <- rbind(raw_model,
#                       corr_model)
# 
# # create evaluation statistics
# print("Example of the statistics for one station: ")
# Summary_stats <- openair::modStats(mod_obs_data_1, obs = "obs", mod = "mod", type = "model")
# print(Summary_stats)


