############################################################
#                                                          #
#              Temporal variations                         #
#                                                          #
############################################################
#-------------------------------------------------------------------------------
# Plot per la mitjana de totes les estacions:

#First, we create a matrix to put the data of observations and model (corrected and raw)
obs_total          <- matrix(nrow = length(date_times$date), ncol = 0) 
raw_model_total    <- matrix(nrow = length(date_times$date), ncol = 0)
corr_model_total   <- matrix(nrow = length(date_times$date), ncol = 0)

for (i in 1:nrow(local_stations)) {
  obs_estacio         <-  df_obs_sub[ ,colnames(llista_uk_total)[i]]
  raw_model_estacio   <-  llista_uk_total_df[ ,colnames(llista_uk_total)[i]]
  corr_model_estacio  <-  LOOCV_total_df[ ,colnames(llista_uk_total)[i]]
  
  obs_total        <- cbind(obs_total,obs_estacio)
  raw_model_total  <- cbind(raw_model_total,raw_model_estacio)
  corr_model_total <- cbind(corr_model_total,corr_model_estacio)
}

#Then, we calculate the means of each hour considering the whole stations
raw_model <- data.frame( obs = rowMeans(obs_total,na.rm=TRUE),
                         mod = rowMeans(raw_model_total,na.rm=TRUE),
                         model="raw_model")

corr_model <- data.frame( obs = rowMeans(obs_total,na.rm=TRUE),
                          mod = rowMeans(corr_model_total,na.rm=TRUE),
                          model="corr_model")

Mean_all_stations            <- data.frame(date            = date_times,
                                           observations       = raw_model$obs,
                                           Model              = raw_model$mod, 
                                           Model_corrected    = corr_model$mod)

TIME_VAR <- openair::timeVariation(Mean_all_stations,
                                   pollutant = c("observations","Model","Model_corrected"),
                                   ylab = "no2 (ug/m3)",
                                   type ="month",
                                   key = TRUE,
                                   title = "Media horaria Cat",
                                   auto.text=FALSE, 
                                   ylim = c(0, 120),
                                   cols = c("black", "blue", "grey"),
                                   lty = c("dotted","solid","dashed"),
                                   lwd = c(3,2,2), 
                                   name.pol = c("Observations", "Raw model", "Corrected Model"))  

png(file = paste0(save_general_statistics,"temporal","_mean_time_VAR.png"), width = 800, height = 600)
print(TIME_VAR[[1]][2]) 
dev.off()
#-------------------------------------------------------------------------------
# Plot per cada estació:


for (n in 1:nrow(local_stations)) {
  #ref_names <- 
  
  Single_station            <- data.frame(date               = date_times,
                                          observations       = df_obs_sub[ ,colnames(llista_uk_total)[n]],
                                          Model              = llista_uk_total_df[ ,colnames(llista_uk_total)[n]], 
                                          Model_corrected    = LOOCV_total_df[ ,colnames(llista_uk_total)[n]])
  
  TIME_VAR <- openair::timeVariation(Single_station,
                                     pollutant = c("observations","Model","Model_corrected"),
                                     ylab = "no2 (ug/m3)",
                                     type ="month",
                                     key = TRUE,
                                     title = colnames(llista_uk_total)[n],
                                     auto.text=FALSE, 
                                     ylim = c(0, 120),
                                     cols = c("black", "blue", "grey"),
                                     lty = c("dotted","solid","dashed"),
                                     lwd = c(3,2,2), 
                                     name.pol = c("observations","Raw model", "Corrected Model"))  
  
  png(file = paste0(save_temporal_variation,local_stations$ref_obs[n],"_", local_stations$station_name[n],"_time_VAR.png"), width = 800, height = 600)
  print(TIME_VAR[[1]][2]) 
  dev.off()
  
}

